import _extends from "@babel/runtime/helpers/esm/extends";
import L from 'leaflet';
import Polygons from './shapes/Polygons';
import Circle from './shapes/Circle';
import { mapViewModes } from './constants';
import MarkerManager from './marker-manager';
import LayerManager from './layer-manager';

var LeafletMap = /*#__PURE__*/function () {
  function LeafletMap(properties) {
    this.setMapDOMInstance(properties.mapDOMInstance);
    this.createMarkerManager(properties.mapDOMInstance);
    this.createLayerManager(properties.id);
    this.buildMap(properties);
    this.setZoomControlPosition(properties.zoomControlPosition);
    this.buildShapes(properties);
    this.subscribeToLeafletMapEvents();
    this.layerManager.addChangeViewController(properties, this._map, this._normalViewText, this._satelliteViewText);
    this.markerManager.addIconMarkersToMap({
      icons: properties.icons,
      map: this._map
    });
    this.dispatchFirstLoad();
  }

  var _proto = LeafletMap.prototype;

  _proto.buildMap = function buildMap(properties) {
    this.layerManager.createMapLayers(properties);
    var mapOptions = {
      center: [properties.latitude, properties.longitude],
      dragging: properties.dragging,
      layers: [this.layerManager.layers.map[properties.selectedMapViewMode]],
      scrollWheelZoom: properties.scrollWheelZoom,
      zoom: properties.zoom,
      zoomControl: properties.zoomControl
    };
    this._map = L.map(properties.id, mapOptions);
    this.attachPropsToMapInstance(properties);
  };

  _proto.dispatchFirstLoad = function dispatchFirstLoad() {
    this.dispatchCustomEvent({
      eventName: 'leaflet_map_loaded',
      detail: this.getParamsForRequest()
    });
  };

  _proto.buildShapes = function buildShapes(properties) {
    properties.polygons && this.buildPolygons(_extends({}, properties));
    properties.radius && this.buildCircle(_extends({}, properties));
  };

  _proto.setMapDOMInstance = function setMapDOMInstance(mapDOMInstance) {
    this.mapDOM = mapDOMInstance;
  };

  _proto.buildPolygons = function buildPolygons(_ref) {
    var currentGeoCode = _ref.currentGeoCode,
        hoverStyles = _ref.hoverStyles,
        onLayerClick = _ref.onLayerClick,
        onPolygonWithBounds = _ref.onPolygonWithBounds,
        polygons = _ref.polygons,
        showLabels = _ref.showLabels;
    this.polygons = new Polygons({
      currentGeoCode: currentGeoCode,
      hoverStyles: hoverStyles,
      onLayerClick: onLayerClick,
      onPolygonWithBounds: onPolygonWithBounds,
      showLabels: showLabels
    });
    this.polygons.setPolygonsOnMap({
      map: this._map,
      polygons: polygons
    });
  };

  _proto.buildCircle = function buildCircle(_ref2) {
    var latitude = _ref2.latitude,
        longitude = _ref2.longitude,
        radius = _ref2.radius,
        onPolygonWithBounds = _ref2.onPolygonWithBounds;
    this.circle = new Circle({
      latitude: latitude,
      longitude: longitude,
      radius: radius,
      onPolygonWithBounds: onPolygonWithBounds
    });
    this.circle.setCircleOnMap({
      map: this._map
    });
  };

  _proto.createMarkerManager = function createMarkerManager(mapId) {
    this.markerManager = new MarkerManager(mapId);
  };

  _proto.createLayerManager = function createLayerManager() {
    this.layerManager = new LayerManager();
  };

  _proto.showHeatMap = function showHeatMap(url) {
    this.layerManager.addHeatMapLayer(url, this._map);
  };

  _proto.removeHeatMapLayer = function removeHeatMapLayer() {
    this._map.removeLayer(this.layerManager.layers.heatMap);
  };

  _proto.setZoomControlPosition = function setZoomControlPosition(position) {
    this._map.zoomControl && this._map.zoomControl.setPosition(position);
  };

  _proto.attachPropsToMapInstance = function attachPropsToMapInstance(properties) {
    this._map.props = properties;
  };

  _proto.setCenter = function setCenter(options) {
    this.setCenter(L.latLng(parseFloat(options.latitude), parseFloat(options.longitude)));
    options.zoom && this.setZoom(options.zoom);
  };

  _proto.getBounds = function getBounds() {
    return {
      northEast: this._map.getBounds().getNorthEast(),
      northWest: this._map.getBounds().getNorthWest(),
      southEast: this._map.getBounds().getSouthEast(),
      southWest: this._map.getBounds().getSouthWest()
    };
  };

  _proto.getZoomLevel = function getZoomLevel() {
    return this._map.getZoom();
  };

  _proto.getCenter = function getCenter() {
    return {
      latitude: this._map.getCenter().lat,
      longitude: this._map.getCenter().lng
    };
  };

  _proto.getCenterWithZoom = function getCenterWithZoom() {
    return _extends({}, this.getCenter(), {
      zoom: this.getZoomLevel()
    });
  };

  _proto.clearLayer = function clearLayer(layer) {
    layer.clearLayers();

    this._map.removeLayer(layer);
  };

  _proto.addLayerGroupToMap = function addLayerGroupToMap(layer) {
    this._map.addLayer(layer);
  };

  _proto.invalidateSize = function invalidateSize(effect) {
    this._map.invalidateSize(effect);
  };

  _proto.setView = function setView(viewType) {
    if (viewType === mapViewModes.NORMAL && this._currentLayer !== mapViewModes.NORMAL) {
      this._map.removeLayer(this.layerManager.layers.map[1]);

      this._map.addLayer(this.layerManager.layers.map[0]);

      this._currentLayer = viewType;
    } else if (viewType === mapViewModes.SATELLITE && this._currentLayer !== mapViewModes.SATELLITE) {
      this._map.removeLayer(this.layerManager.layers.map[0]);

      this._map.addLayer(this.layerManager.layers.map[1]);

      this._currentLayer = viewType;
    }
  };

  _proto.subscribeToLeafletMapEvents = function subscribeToLeafletMapEvents() {
    var _this = this;

    this._map.on('load', function (e) {
      _this.dispatchCustomEvent({
        eventName: 'leaflet_map_load',
        detail: _this.getParamsForRequest()
      });
    });

    this._map.on('click', function (e) {
      _this.dispatchCustomEvent({
        eventName: 'leaflet_map_click',
        detail: _this.getParamsForRequest()
      });
    });

    this._map.on('dragend', function (e) {
      _this.dispatchCustomEvent({
        eventName: 'leaflet_map_dragend',
        detail: _this.getParamsForRequest()
      });
    });

    this._map.on('zoomend', function (e) {
      _this.dispatchCustomEvent({
        eventName: 'leaflet_map_zoomend',
        detail: _this.getParamsForRequest()
      });
    });

    this._map.on('baselayerchange', function (e) {
      if (e.name === _this._tileLayerNormal) {
        _this.dispatchCustomEvent({
          eventName: 'leaflet_map_layer_normal',
          detail: _this.getParamsForRequest()
        });
      }
    });

    this._map.on('baselayerchange', function (e) {
      if (e.name === _this._tileLayerSatName) {
        _this.dispatchCustomEvent({
          eventName: 'leaflet_map_layer_satellite',
          detail: _this.getParamsForRequest()
        });
      }
    });
  };

  _proto.getMapBoundingBox = function getMapBoundingBox() {
    var _this$getBounds = this.getBounds(),
        northWest = _this$getBounds.northWest,
        southEast = _this$getBounds.southEast;

    return northWest.lng + ',' + northWest.lat + ';' + northWest.lng + ',' + southEast.lat + ';' + southEast.lng + ',' + southEast.lat + ';' + southEast.lng + ',' + northWest.lat + ';' + northWest.lng + ',' + northWest.lat;
  };

  _proto.getParamsForRequest = function getParamsForRequest() {
    var _this$getCenterWithZo = this.getCenterWithZoom(this._map),
        zoom = _this$getCenterWithZo.zoom,
        latitude = _this$getCenterWithZo.latitude,
        longitude = _this$getCenterWithZo.longitude;

    return {
      latitude: latitude,
      longitude: longitude,
      mapBoundingBox: this.getMapBoundingBox(),
      zoomLevel: zoom
    };
  };

  _proto.clearMarkersLayer = function clearMarkersLayer() {
    this.layerManager.layers.markers && this.clearLayer(this.layerManager.layers.markers);
  } // Returns the elements in arrayToCompare that are not present in the arrayOrigin
  ;

  _proto.getPositiveDiffOfArraysOfPoints = function getPositiveDiffOfArraysOfPoints(_ref3) {
    var pois = _ref3.pois,
        currentMarkers = _ref3.currentMarkers;
    return pois.reduce(function (accumulate, originalArrayPoint) {
      !currentMarkers.some(function (comparedArrayPoint) {
        return comparedArrayPoint.Id === originalArrayPoint.Id && comparedArrayPoint.options.icon.options.className.includes(originalArrayPoint.customClassName);
      }) && accumulate.push(originalArrayPoint);
      return accumulate;
    }, []);
  };

  _proto.getPositiveDiffOfArraysOfPointsAux = function getPositiveDiffOfArraysOfPointsAux(_ref4) {
    var pois = _ref4.pois,
        currentMarkers = _ref4.currentMarkers;
    return currentMarkers.reduce(function (accumulate, originalArrayPoint) {
      !pois.some(function (comparedArrayPoint) {
        return comparedArrayPoint.Id === originalArrayPoint.Id && originalArrayPoint.options.icon.options.className.includes(comparedArrayPoint.customClassName);
      }) && accumulate.push(originalArrayPoint);
      return accumulate;
    }, []);
  };

  _proto.dispatchCustomEvent = function dispatchCustomEvent(_ref5) {
    var eventName = _ref5.eventName,
        detail = _ref5.detail;
    var event;

    if (this.mapDOM.CustomEvent && typeof this.mapDOM.CustomEvent === 'function') {
      event = new this.mapDOM.CustomEvent(eventName, {
        detail: detail
      });
    } else {
      event = document.createEvent('CustomEvent');
      event.initCustomEvent(eventName, true, true, detail);
    }

    this.mapDOM.dispatchEvent(event);
  };

  _proto.addLayersToMap = function addLayersToMap(layerDataArray, groupName) {
    var _this2 = this;

    var layers = layerDataArray.map(function (layerData) {
      return _this2.markerManager.createMarker(layerData);
    });
    this.layerManager.addLayersToGroup(layers, groupName);
    this.addLayerGroupToMap(this.layerManager.layers[groupName]);
  } // We should check if there's a better solution for cluster poi's like the use of plugins http://leafletjs.com/plugins.html#clusteringdecluttering
  ;

  _proto.displayPois = function displayPois(pois) {
    if (!pois || !pois.length) {
      this.clearMarkersLayer();
    } else {
      // Get new Marker Type by checking one new POI.
      var markerType = pois[0].markerType;
      var shouldClearAllMarkers = markerType !== this.markerManager.currentMarkerType; // If we render a new type of Marker, then we need to clear all Markers.

      shouldClearAllMarkers && this.clearMarkersLayer(); // Get current array of map Markers.

      var currentMarkers = this.layerManager.getLayers(this.layerManager.layers.markers); // To prevent repaint all POIs that are already visible, get the news to Add and the ones to be Removed.

      var pointsToAdd = this.getPositiveDiffOfArraysOfPoints({
        pois: pois,
        currentMarkers: currentMarkers
      });
      var pointsToDelete = this.getPositiveDiffOfArraysOfPointsAux({
        currentMarkers: currentMarkers,
        pois: pois
      });
      var pointsAreClickable = markerType > 0; // Keep the current lat/lng of a POI that is going to be repaint due to selection with a new random lat/lng.
      // In order to prevent that POIs change its lat/lon for de-clusterized properties.
      // markerType > 0 means that POIs are clickable, so they can be selected, and for instance, repaint.

      pointsToDelete.length && pointsAreClickable && pointsToAdd.forEach(function (pointToAdd) {
        var pointToRefresh = pointsToDelete.find(function (pointToDelete) {
          return pointToDelete.Id === pointToAdd.Id;
        });

        if (pointToRefresh) {
          pointToAdd.latitude = pointToRefresh._latlng.lat;
          pointToAdd.longitude = pointToRefresh._latlng.lng;
        }
      }); // Set new markerType (0, 1, 2).

      this.markerManager.resetMarkerType(markerType);
      this.markerManager.currentMarkerType = markerType; // Remove pointsToDelete, and Add new POI's to map.

      pointsToDelete.length && this.layerManager.removeLayersFromGroup(pointsToDelete, 'markers');
      pointsToAdd.length && this.addLayersToMap(pointsToAdd, 'markers');
    }
  };

  return LeafletMap;
}();

export { LeafletMap as default };